package reflection.junitsimulaator;

import java.lang.reflect.*;

public class JUnitSimulator {
    public static void main(String[] args) throws Exception {
        Class<?> testClass = CalculatorTest.class;

        Object testInstance = testClass.getDeclaredConstructor().newInstance();

        Method[] methods = testClass.getDeclaredMethods();

        int testsRun = 0;
        int testsPassed = 0;

        for (Method method : methods) {
            if (method.isAnnotationPresent(Test.class)) {
                Test testAnnotation = method.getAnnotation(Test.class);
                Class<? extends Throwable> expectedException = testAnnotation.expected();

                try {
                    System.out.println("Running test: " + method.getName());
                    method.invoke(testInstance);
                    testsRun++;

                    if (expectedException != Test.None.class) {
                        System.out.println("Test failed: Expected exception " +
                                expectedException.getSimpleName() +
                                " but none was thrown");
                    } else {
                        testsPassed++;
                        System.out.println("Test passed");
                    }
                } catch (InvocationTargetException e) {
                    Throwable actualException = e.getCause();

                    if (expectedException.isInstance(actualException)) {
                        testsRun++;
                        testsPassed++;
                        System.out.println("Test passed (expected exception caught)");
                    } else {
                        testsRun++;
                        System.out.println("Test failed: Unexpected exception " +
                                actualException.getClass().getSimpleName());
                    }
                } catch (Exception e) {
                    testsRun++;
                    System.out.println("Test failed with error: " + e.getMessage());
                }
            }
        }

        System.out.println("\nTest results: " + testsPassed + "/" + testsRun + " passed");
    }
}

