package minecraft.player;

import minecraft.item.Food;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.Arguments;
import org.junit.jupiter.params.provider.MethodSource;

import java.util.stream.Stream;

import static org.junit.jupiter.api.Assertions.*;

class SteveTest {
    Steve steve;

    @BeforeEach
    void initSteve() throws InvalidCharacterParamsException {
        steve = new Steve("Juraj", 1, 10, 2, 1, 1, 11, false);
    }

    @Test
    void constructorNegativeHungerTest() throws InvalidCharacterParamsException {
        assertThrows(InvalidCharacterParamsException.class, () -> new Steve("Juraj", 1, 1, 1, 1, -1, 1, false));
    }

    @Test
    void constructorNegativeMaxHungerTest() throws InvalidCharacterParamsException {
        assertThrows(InvalidCharacterParamsException.class, () -> new Steve("Juraj", 1, 1, 1, 1, 1, -1, false));
    }

    @Test
    void constructorLargerThanMaxHungerTest() throws InvalidCharacterParamsException {
        assertThrows(InvalidCharacterParamsException.class, () -> new Steve("Juraj", 1, 1, 1, 1, 2, 1, false));
    }

    @Test
    void consumeFoodHungerSimpleTest() {
        Food food = new Food("Jablko", 1, 1, 5, 4);

        steve.consume(food);

        assertEquals(5, steve.getHunger());
    }

    @Test
    void consumeFoodZeroTest() {
        Food food = new Food("Jablko", 1, 1, 5, 10);
        food.setAmount(0);

        steve.consume(food);

        assertEquals(1, steve.getHealth());
    }

    @Test
    void consumeFoodMaxHungerTest() {
        Food food = new Food("Jablko", 1, 1, 10, 22);

        steve.consume(food);

        assertEquals(11, steve.getHunger());
    }

    @Test
    void attackSteveTest() throws InvalidCharacterParamsException {
        Steve steve2 = new Steve("AAAA", 1, 1, 1, 1, 1, 1, false);

        steve.attack(steve2);

        assertEquals(0, steve2.getHealth());
    }

    @Test
    void attackSteveGodTest() throws InvalidCharacterParamsException {
        steve.setGodMode(true);
        Steve steve2 = new Steve("AAAA", 10, 10, 1, 1, 1, 1, false);

        steve.attack(steve2);

        assertEquals(0, steve2.getHealth());
    }

    @Test
    void attackSteveGodGodTest() throws InvalidCharacterParamsException {
        steve.setGodMode(true);
        Steve steve2 = new Steve("AAAA", 10, 10, 1, 1, 1, 1, true);

        steve.attack(steve2);

        assertEquals(10, steve2.getHealth());
    }

    @Test
    void attackNPC() throws InvalidCharacterParamsException {
        NPC npc = new NPC("NPC", 1, 1, 1, 1);

        steve.attack(npc);

        assertEquals(0, npc.getHealth());
    }

    @Test
    void attackNPCGodTest() throws InvalidCharacterParamsException {
        steve.setGodMode(true);
        NPC npc = new NPC("NPC", 10, 10, 1, 1);

        steve.attack(npc);

        assertEquals(0, npc.getHealth());
    }


    @Test
    void attackCreeper() throws InvalidCharacterParamsException {
        Creeper creeper = new Creeper("Creeper", 1, 1, 1, 1, 1.5);

        steve.attack(creeper);

        assertEquals(0, creeper.getHealth());
    }

    @Test
    void attackCreeperGodTest() throws InvalidCharacterParamsException {
        steve.setGodMode(true);
        Creeper creeper = new Creeper("Creeper", 10, 10, 1, 1, 1.5);

        steve.attack(creeper);

        assertEquals(0, creeper.getHealth());
    }


    @Test
    void equalsTest() throws InvalidCharacterParamsException {
        Steve steve1 = new Steve("Egres", 1, 1, 1, 1, 1, 1, false);
        Steve steve2 = new Steve("Egres", 1, 1, 1, 1, 1, 1, false);

        assertEquals(steve1, steve2);
    }

    @Test
    void equalsDiffTest() throws InvalidCharacterParamsException {
        Steve steve1 = new Steve("Egres", 1, 1, 1, 1, 1, 1, false);
        Steve steve2 = new Steve("Egres", 2, 3, 1, 1, 1, 1, false);

        assertNotEquals(steve1, steve2);
    }


    @Test
    void equalsSameObjectTest() throws InvalidCharacterParamsException {
        Steve steve1 = new Steve("Egres", 1, 1, 1, 1, 1, 1, false);

        assertEquals(steve1, steve1);
    }

    @Test
    void equalsDiffTypesTest() throws InvalidCharacterParamsException {
        Steve steve1 = new Steve("Egres", 1, 1, 1, 1, 1, 1, false);
        NPC steve2 = new NPC("Egres", 2, 3, 1, 1);

        assertNotEquals(steve1, steve2);
    }


    @Test
    void hashCodeTest() throws InvalidCharacterParamsException {
        Steve steve1 = new Steve("Egres", 1, 1, 1, 1, 1, 1, false);
        Steve steve2 = new Steve("Egres", 1, 1, 1, 1, 1, 1, false);

        assertEquals(steve1.hashCode(), steve2.hashCode());
    }

    @Test
    void hashCodeDifferentTest() throws InvalidCharacterParamsException {
        Steve steve1 = new Steve("Egres", 1, 1, 1, 1, 1, 1, false);
        Steve steve2 = new Steve("Egres", 1, 2, 1, 1, 1, 1, false);

        assertNotEquals(steve1.hashCode(), steve2.hashCode());
    }

    @Test
    void compareToEqualNameTest() throws InvalidCharacterParamsException {
        Steve steve1 = new Steve("Egres", 1, 1, 1, 1, 1, 1, false);
        Steve steve2 = new Steve("Egres", 1, 1, 1, 1, 1, 1, false);

        assertEquals(0, steve1.compareTo(steve2));
    }

    @Test
    void compareToLowerNameTest() throws InvalidCharacterParamsException {
        Steve steve1 = new Steve("Egres", 1, 1, 1, 1, 1, 1, false);
        Steve steve2 = new Steve("Jurko", 1, 1, 1, 1, 1, 1, false);

        assertTrue(steve1.compareTo(steve2) > 0);
    }

    @Test
    void compareToHigherNameTest() throws InvalidCharacterParamsException {
        Steve steve1 = new Steve("Jurko", 1, 1, 1, 1, 1, 1, false);
        Steve steve2 = new Steve("Egres", 1, 1, 1, 1, 1, 1, false);

        assertTrue(steve1.compareTo(steve2) < 0);
    }

    @ParameterizedTest
    @MethodSource("dataGenerator")
    void testParameterizedComputeIterative(Steve steve2) throws Exception {
        assertNotEquals(steve, steve2);
    }

    private static Stream<Arguments> dataGenerator() throws InvalidCharacterParamsException {
        return Stream.of(
                Arguments.of(new Steve("Jurajj", 1, 10, 2, 1, 1, 11, false)),
                Arguments.of(new Steve("Juraj", 2, 10, 2, 1, 1, 11, false)),
                Arguments.of(new Steve("Juraj", 1, 9, 2, 1, 1, 11, false)),
                Arguments.of(new Steve("Juraj", 1, 10, 1, 1, 1, 11, false)),
                Arguments.of(new Steve("Juraj", 1, 10, 2, 2, 1, 11, false)),
                Arguments.of(new Steve("Juraj", 1, 10, 2, 1, 2, 11, false)),
                Arguments.of(new Steve("Juraj", 1, 10, 2, 1, 1, 10, false)),
                Arguments.of(new Steve("Juraj", 1, 10, 2, 1, 1, 11, true))
        );
    }
}